@@ -1,19 +1,47 @@
+/*
+ * dvi2tty
+ * Copyright (C) 2003 Marcel J.E. Mol <marcel@mesa.nl>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ */
+
 #define Copyright "dvi2tty  Copyright (C) 1984, 1985, 1986 Svante Lindahl.\n\
-Copyright (C) 1988 M.J.E. Mol 1989-1995"
+Copyright (C) 1989-1996 M.J.E. Mol"
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #if defined(MSDOS)
-#include <malloc.h>
+# include <malloc.h>
 #else
-#include <unistd.h>
+# if !defined(THINK_C)
+#  include <unistd.h>
+# endif
 #endif
 
 #define TRUE        1
 #define FALSE       0
 #define nil         NULL
 
+/*
+ * Define the maximum width of the terminal
+ * (this is also used to define the with of the line structure in dvistuff.c)
+ */
+#define MAXTERMWIDTH	332
+
 /*
  * ERROR CODES , don't start with 0
  */
@@ -70,6 +98,8 @@ extern bool   accent;                  /* Output accent stuff(dvistuff.c)    */
 extern bool   ttfont;                  /* tt font assumed   (dvistuff.c)     */
 extern bool   japan;                   /* japanes font support (dvistuff.c)  */
 extern bool   noffd;                   /* output ^L or formfeed (dvistuff.c) */
+extern bool   printfont;               /* include font switches (dvistuff.c) */
+extern bool   allchar;                 /* output all characters (dvistuff.c  */
 
 extern printlisttype *currentpage;     /* current page to print (dvi2tty.c)  */
 extern printlisttype *firstpage;       /* first page selected (dvi2tty.c)    */
@@ -80,6 +110,7 @@ extern int            espace;          /* extra screen width (dvi2tty.c)     */
 extern long           foo;             /* temporary 'register' (dvi2tty.c)   */
 extern long           lineheight;      /* height of a line (dvistuff.c)      */
 extern int            opcode;          /* dvi opcode (dvistuff.c)            */
+extern char *         delim;           /* printer font delim (dvistuff.c)    */
 
 extern FILE *DVIfile;                  /* dvi file (dvi2tty.c)               */
 extern FILE *output;                   /* output file (dvi2tty.c)            */
