@@ -1,3 +1,23 @@
+/*
+ * dvi2tty
+ * Copyright (C) 2003 Marcel J.E. Mol <marcel@mesa.nl>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *
+ */
+
 /*
  * Include files
  */
@@ -5,15 +25,19 @@
 #include "dvi2tty.h"
 
 #if defined(VMS) 
-#include types.h
-#include stat
+# include types.h
+# include stat
 #else
-#include <sys/types.h>
-#include <sys/stat.h>
+# if defined(THINK_C)
+#  include <unix.h>
+# else
+#  include <sys/types.h>
+#  include <sys/stat.h>
+# endif
 #endif
 
-#if defined(MSDOS)
-#include <math.h>
+#if defined(MSDOS) || defined(THINK_C)
+# include <math.h>
 #endif
 
 #include "commands.h"
@@ -34,9 +58,10 @@
 #define VERSIONID            2 /* dvi version number that pgm handles      */
 #define VERTICALEPSILON 450000L /* crlf when increasing v more than this   */
 
-#define rightmargin     152    /* nr of columns allowed to the right of h=0*/
+#define rightmargin     MAXTERMWIDTH+20 
+                               /* nr of columns allowed to the right of h=0*/
 #define leftmargin      -50    /* give some room for negative h-coordinate */
-#define LINELEN         203    /* rightmargin - leftmargin + 1 */
+#define LINELEN         rightmargin - leftmargin + 1 
 
 #define MOVE            TRUE   /* if advancing h when outputing a rule     */
 #define STAY            FALSE  /* if not advancing h when outputing a rule */
@@ -68,7 +93,7 @@
  * Structure and variable definitions
  */
 
-char *dvistuff = "@(#) dvistuff.c  5.10 23jul95 M.J.E. Mol (c) 1989-1995";
+char *dvistuff = "@(#) dvistuff.c  5.12 27may1996 M.J.E. Mol (c) 1989-1996";
 
 typedef struct {
     long hh;
@@ -105,6 +130,10 @@ bool        japan = FALSE;      /* switch to japanese fonts ...              */
 bool        mifont = FALSE;
 int         jfontnum = 0;
 bool        noffd;              /* if true output ^L instead of formfeed     */
+char *      delim;              /* -bdelim for font switch printing          */
+bool        printfont;          /* true if user wants font switches printed  */
+bool        allchar;            /* true if user sets all characters          */
+                                /* overrides sscasci, accent                 */
 
 int         opcode;             /* dvi-opcodes                               */
 
@@ -231,10 +260,17 @@ void postamble(void)
 {
     register long size;
     register int  count;
+#if !defined (THINK_C)
     struct stat st;
+#endif
 
+#if defined (THINK_C)
+    size = DVIfile->len;
+#else
     fstat (fileno(DVIfile), &st);
     size = (long) st.st_size;                   /* get size of file          */
+#endif
+
     count = -1;
     do {              /* back file up past signature bytes (223), to id-byte */
         if (size-- == 0)
@@ -377,7 +413,7 @@ void dopage(void)
         if (opcode <= LASTCHAR)
             dochar((char) opcode);
         else if ((opcode >= FONT_00) && (opcode <= FONT_63)) 
-            setfont(opcode - FONT_00);
+            setfont((long) opcode - FONT_00);
         else if (opcode > POST_POST)
             errorexit(illop);
         else
@@ -569,7 +605,7 @@ void printpage(void)
             for (i = firstcolumn, j = 1; i <= currentline->charactercount;
                    i++, j++) {
                 ch = currentline->text[i - leftmargin];
-                if (ch >= SPACE)
+                if (ch >= SPACE || allchar)
                     putc(ch, output);
                 if ((j > (int) foo) && (currentline->charactercount > i+1)) {
                         fprintf(output, "*\n");         /* if line to large */
@@ -632,7 +668,6 @@ void rule(bool moving, long rulewt, long ruleht)
     register char ch;               /* character to set rule with            */
     register long saveh, savev;
                               /* rule   --   starts up the recursive routine */
-    saveh = h;			/* avoid compile error (branderh@debian.org) */
     if (!moving)
         saveh = h;
     if ((ruleht <= 0) || (rulewt <= 0))
@@ -698,7 +733,7 @@ void ruleaux(long rulewt, long ruleht, char ch)
 long horizontalmove(long amount)
 {
 
-#if defined(MSDOS)
+#if defined(MSDOS) || defined(THINK_C)
     if (labs(amount) > charwidth / 4L) {    /* } to make vi happy */
 #else
     if (abs(amount) > charwidth / 4L) {
@@ -828,7 +863,7 @@ linetype *findline(void)
 unsigned long num(int size)
 {
     register int i;
-    register long x = 0;
+    register unsigned long x = 0;
 
     for (i = size; i > 0; i--)
         x = (x << 8) + (unsigned) getc(DVIfile);
@@ -845,7 +880,7 @@ unsigned long num(int size)
 long snum(int size)
 {
     register int i;
-    register long x = 0;
+    register long x;
 
     x = getc(DVIfile);
     if (x & 0x80)
@@ -867,7 +902,7 @@ void dochar(char ch)
 {
  
     if (japan && jfontnum)
-        jischar(ch);
+        jischar((long) ch);
     else if (symbolfont == TRUE)
         symchar(ch);
     else
@@ -923,75 +958,102 @@ void normchar(char ch)
     switch (ch) {
         case 11  :  if (ttfont) 
                         ch = '^';                   /* up symbol       */
-                    else {
+                    else if (!allchar) {
                         outchar('f'); ch = 'f';     /* ligature        */
                     }
                     break;
         case 12  :  if (ttfont) 
                         ch = 'v';                   /* low symbol       */
-                    else {
+                    else if (!allchar) {
                         outchar('f'); ch = 'i';     /* ligature        */
                     }
                     break;
         case 13  :  if (ttfont) 
                         ch = '`';
-                    else {
+                    else if (!allchar) {
                         outchar('f'); ch = 'l';     /* ligature        */
                     }
                     break;
         case 14  :  if (ttfont) 
                         ch = 'i';                   /* spanish !        */
-                    else {
+                    else if (!allchar) {
                         outchar('f'); outchar('f');
                                   ch = 'i';         /* ligature        */
                     }
                     break;
         case 15  :  if (ttfont) 
                         ch = '.';                   /* spanish ?        */
-                    else {
+                    else if (!allchar) {
                         outchar('f'); outchar('f');
                                   ch = 'l';         /* ligature        */
                     }
                     break;
-        case 16  :  ch = 'i'; break;
-        case 17  :  ch = 'j'; break;
-        case 25  :  outchar('s'); ch = 's'; break;  /* German double s */
-        case 26  :  outchar('a'); ch = 'e'; break;  /* Dane/Norw ae    */
-        case 27  :  outchar('o'); ch = 'e'; break;  /* Dane/Norw oe    */
-        case 28  :  if (scascii)
-                        ch = '|';                   /* Dane/Norw /o    */
-                    else
-                        ch = 'o';
+        case 16  :  if (!allchar) ch = 'i'; break;
+        case 17  :  if (!allchar) ch = 'j'; break;
+        case 25  :  if (!allchar) {
+                        outchar('s');
+                        ch = 's';
+                    }
+                    break;  /* German double s */
+        case 26  :  if (!allchar) {
+                        outchar('a');
+                        ch = 'e';
+                    }
+                    break;  /* Dane/Norw ae    */
+        case 27  :  if (!allchar) {
+                        outchar('o');
+                        ch = 'e';
+                    }
+                    break;  /* Dane/Norw oe    */
+        case 28  :  if (!allchar) {
+                        if (scascii)
+                            ch = '|';                   /* Dane/Norw /o    */
+                        else
+                            ch = 'o';
+                    }
                     break;
-        case 29  :  outchar('A'); ch = 'E'; break;  /* Dane/Norw AE    */
-        case 30  :  outchar('O'); ch = 'E'; break;  /* Dane/Norw OE    */
-        case 31  :  if (scascii)
-                        ch = '\\';                  /* Dane/Norw /O    */
-                    else
-                        ch = 'O';
+        case 29  :  if (!allchar) {
+                        outchar('A');
+                        ch = 'E';
+                    }
+                    break;  /* Dane/Norw AE    */
+        case 30  :  if (!allchar) {
+                        outchar('O');
+                        ch = 'E';
+                    }
+                    break;  /* Dane/Norw OE    */
+        case 31  :  if (!allchar) {
+                        if (scascii)
+                            ch = '\\';                  /* Dane/Norw /O    */
+                        else
+                            ch = 'O';
+                    }
                     break;
-        case 32  :  ch = ttfont ? ch : '_'; break;  /* underlined blank */
-        case 58  :  ch = mifont ? '.' : ch; break;  /* if japan */
-        case 59  :  ch = mifont ? ',' : ch; break;  /* if japan */
-        case 92  :  ch = ttfont ? ch : '"'; break;  /* \ from `` */
-        case 123 :  ch = ttfont ? ch : '-'; break;  /* { from -- */
-        case 124 :  ch = ttfont ? ch : '_'; break;  /* | from --- */
-        case 125 :  ch = ttfont ? ch : '"'; break;  /* } from \H */
-        case 126 :  ch = ttfont ? ch : '"'; break;  /* ~ from \~ */
-        case 127 :  ch = '"'; break;  /* DEL from \" */
+        case 32  :  ch = allchar || ttfont ? ch : '_'; break;
+                                                        /* underlined blank */
+        case 58  :  ch = allchar || !mifont ? ch : '.'; break; /* if japan */
+        case 59  :  ch = allchar || !mifont ? ch : ','; break; /* if japan */
+        case 92  :  ch = allchar || ttfont ? ch : '"'; break;  /* \ from `` */
+        case 123 :  ch = allchar || ttfont ? ch : '-'; break;  /* { from -- */
+        case 124 :  ch = allchar || ttfont ? ch : '_'; break;  /* | from --- */
+        case 125 :  ch = allchar || ttfont ? ch : '"'; break;  /* } from \H */
+        case 126 :  ch = allchar || ttfont ? ch : '"'; break;  /* ~ from \~ */
+        case 127 :  if (!allchar) ch = '"'; break;             /* DEL from \" */
        
         /*
          * Or should I use SPACE for non-accents ???
          * This seems to work ...
          */
-        case 18  :  ch = accent ? '`' : ch; break;  /* from \` */
-        case 19  :  ch = accent ? 0x27 : ch; break; /* from \' */
-        case 20  :  ch = accent ? '~' : ch; break;  /* from \v */
-        case 21  :  ch = accent ? '~' : ch; break;  /* from \u */
-        case 22  :  ch = accent ? '~' : ch; break;  /* from \= */
-        case 24  :  ch = accent ? ',' : ch; break;  /* from \c */
-        case 94  :  ch = (accent && !ttfont) ? '^' : ch; break;  /* ^ from \^ */
-        case 95  :  ch = (accent && !ttfont) ? '`' : ch; break;  /* _ from \. */
+        case 18  :  ch = !allchar && accent ? '`' : ch; break;  /* from \` */
+        case 19  :  ch = !allchar && accent ? 0x27 : ch; break; /* from \' */
+        case 20  :  ch = !allchar && accent ? '~' : ch; break;  /* from \v */
+        case 21  :  ch = !allchar && accent ? '~' : ch; break;  /* from \u */
+        case 22  :  ch = !allchar && accent ? '~' : ch; break;  /* from \= */
+        case 24  :  ch = !allchar && accent ? ',' : ch; break;  /* from \c */
+        case 94  :  ch = (!allchar && accent && !ttfont) ? '^' : ch; break;
+                                                                /* ^ from \^ */
+        case 95  :  ch = (!allchar && accent && !ttfont) ? '`' : ch; break;
+                                                                /* _ from \. */
     }
     outchar(ch); 
 
@@ -1004,7 +1066,7 @@ void normchar(char ch)
 /*
  * OUTCHAR -- Here we put the character into the current page.
  *            This function includes some code to handle Scandinavian
- *            characters. I think that code doesn'y belong here. IT
+ *            characters. I think that code doesn't belong here. IT
  *            SHOULD BE MOVED OUT.
  */
 
@@ -1014,7 +1076,7 @@ void outchar(char ch)
 
 /*     fprintf(stderr, "hor: %ld, ver: %ld\n", h, v); */
 
-#if defined(MSDOS)
+#if defined(MSDOS) || defined(THINK_C)
     if (labs(v - currentline->vv) > lineheight / 2L)
 #else
     if (abs(v - currentline->vv) > lineheight / 2L)
@@ -1046,7 +1108,7 @@ void outchar(char ch)
      * or dots and if one is found the corresponding national char *
      * replaces the special character codes.                       *
      *-------------------------------------------------------------*/
-    if (scascii) {
+    if (!allchar && scascii) {
         if ((ch == 'a') || (ch == 'A') || (ch == 'o') || (ch == 'O')) {
             for (i = IMAX(leftmargin, j-2);
                  i <= IMIN(rightmargin, j+2);
@@ -1086,7 +1148,7 @@ void outchar(char ch)
             j++;
             h += charwidth;
         }
-    if ( ((ch >= SPACE) && (ch != DEL)) ||
+    if ( allchar || ((ch >= SPACE) && (ch != DEL)) ||
          (scascii && (ch == 23)) ) {
           /*  (scascii && (ch == DEL)) )     if VMS ??? */
         if (j < rightmargin)
@@ -1116,8 +1178,8 @@ void putcharacter(long charnr)
 
     saveh = h;
     if (japan)
-        dochar(charnr);
-    else if ((charnr >= 0) && (charnr <= LASTCHAR))
+        dochar((char) charnr);
+    else if (allchar || ((charnr >= 0) && (charnr <= LASTCHAR)))
         outchar((char) charnr);
     else
         setchar(charnr);
@@ -1138,7 +1200,7 @@ void putcharacter(long charnr)
 void setchar(long charnr)
 {
 
-    outchar('#');
+    outchar(allchar ? charnr : '#');
 
     return;
 
@@ -1200,12 +1262,15 @@ void fontdef(int x)
 /*
  * SETFONT -- Switch to specific font. Try to find out if it is a symbol
  *            font.
+ *            Option -c allchar does not pertain to this portion, so symbols
+ *            are still translated.
  */
 
 void setfont(long fntnum)
 {
     font * fnt;
     char * s;
+    char * d;
 
     fnt = fonts;
     while (fnt != NULL && fnt->num != fntnum)
@@ -1220,6 +1285,15 @@ void setfont(long fntnum)
     }
 
     s = fnt->name;
+    d = delim;      /* print delim and font name if -b was chosen */
+    if (printfont) {
+         while (*d !=0) {putcharacter(*d); d++;}
+         d = s;
+         while (*d !=0) {putcharacter(*d); d++;}
+         /* d = strrev(delim); */
+         while (*d !=0) {putcharacter(*d); d++;}
+    }                                      
+
     if (japan)
         jfontnum = getjsubfont(s);
     if (jfontnum == 0) {
@@ -1259,9 +1333,9 @@ void setfont(long fntnum)
 
 void jischar(long charnr)
 {
-    int Ku,Ten;
+    int Ku, Ten;
 
-    compute_jis(jfontnum, charnr, &Ku, &Ten);
+    compute_jis(jfontnum, (int) charnr, &Ku, &Ten);
     outchar(Ku+128);
     outchar(Ten+128);
 
